/*----------------------------------------------------------
* Copyright (c) 2002 by National Semiconductor Corporation
* All rights reserved.
*-----------------------------------------------------------
* File Contents:
*     pci.c
* Project: IMB Driver
*---------------------------------------------------------*/
/*
 * $Log:   //epg-scc/sst/ProjectDatabases/ISM/archives/ISM8x/v80/PI/Windows/imbdrv/pci.c.v  $
 * 
 *    Rev 1.2   09 Aug 2004 21:46:24   ckarandx
 * Added support for ICH6
 * 
 *    Rev 1.1   16 Jun 2004 03:54:50   rmisalx
 * After HCT testing, code is checked in
 * 
 *    Rev 1.0   May 13 2004 14:14:10   jggoodno
 * Initial revision.
 * 
 *    Rev 1.0   14 Oct 2003 06:26:58   sgangadx
 * Initial revision.
 * Revision 1.5  2003/07/29 12:33:20Z  marcela
 * ICH_HR added
 * Revision 1.4  2003/03/12 12:05:28Z  marcela
 * Add AMD
 * Revision 1.3  2003/01/29 10:44:48Z  marcela
 * Add ICH5
 * Revision 1.2  2002/12/17 13:34:56Z  marcela
 * Add (()) to DEBUG expressions
 * Revision 1.1  2002/12/11 06:54:16Z  mks
 * Initial revision
 */

#if defined(WIN32) || defined(WINVER)
	#include <wdm.h>
#endif
#include "imb_os.h"
#include "pci.h"

//
// SearchPci()
//
BOOL PciFind (PULONG ID, PUINT pdev, PUINT pfunc)
{
    ULONG   pci_data;
    BOOL    found = FALSE;
    UCHAR   i, j;
    
    // Loop in order to find PCI device
    for( i=0 ; i<32 ; i++ ){
        for( j=0 ; j<8 ; j++ ){
            pci_data = GetPciDevId( i, j );
            switch (pci_data) {
                case ID_8111:
                case ID_81801:
                case ID_81801AB:
                case ID_82801BA:
                case ID_82801CA:
                case ID_CSB5:
                case ID_CSB6:
                case ID_OSB4:
                case ID_ICH4:
                case ID_ICH5:
				case ID_ICH6: //srini added to support ICH6
                case Hance_Rapids:
         	        found = TRUE;
                    break;
                default:
                    found = FALSE;
                    break;
            }
            if (found == TRUE)
                break;
        }// End of inner For 
        if (found == TRUE)
            break;
    }// End of outer For
    
    (*ID)	= pci_data;
    (*pdev)	= (UINT)i;
    (*pfunc)= (UINT)j;

    return found;
}

ULONG GetPciDevId(UCHAR dev, UCHAR func)
{
    ULONG current_pci_id, pci_config_addr;
    
    pci_config_addr  = 0x80000000 | ((dev<<11)&0xFF00);
    pci_config_addr |= ((func<<8)&0x0700);
    
    current_pci_id = PciReadDWord(pci_config_addr);
    
    return current_pci_id;
}


ULONG PciReadDWord (ULONG deviceId)
{

    ULONG dataValue;

//  ULONG addrReg = PCI_CONFIG_ADDRESS;
//  ULONG dataReg = PCI_CONFIG_DATA;
//  WritePortUlong(addrReg, deviceId);
//  ReadPortUlong(dataReg, &dataValue);

	// Updated by Wipro for Driver Merging Activity for ISM8.0
	// For removing Compilation Errors definations of these calls is changed.
    WritePortUlong(PCI_CONFIG_ADDRESS, deviceId);
    ReadPortUlong(PCI_CONFIG_DATA, &dataValue);
    
    return dataValue;

}

VOID PciWriteDWord(ULONG deviceId, ULONG data)
{

//  ULONG addrReg = PCI_CONFIG_ADDRESS;
//  ULONG dataReg = PCI_CONFIG_DATA;
//  WritePortUlong(addrReg, deviceId);
//  WritePortUlong(dataReg, data);

	// Updated by Wipro for Driver Merging Activity for ISM8.0
	// For removing Compilation Errors definations of these calls is changed.
    WritePortUlong(PCI_CONFIG_ADDRESS, deviceId);
    WritePortUlong(PCI_CONFIG_DATA, data);
}

