/*----------------------------------------------------------
* Copyright (c) 2002 by National Semiconductor Corporation
* All rights reserved.
*-----------------------------------------------------------
* File Contents:
*     smb.c
* Project: IMB Driver
*---------------------------------------------------------*/
/*
 * $Log:   //epg-scc/sst/ProjectDatabases/ISM/archives/ISM8x/v80/PI/Windows/imbdrv/smb.c.v  $
 * 
 *    Rev 1.3   09 Aug 2004 21:46:40   ckarandx
 * Added support for ICH6
 * 
 *    Rev 1.2   06 Jul 2004 03:49:34   rmisalx
 * Removed the code that updates the SDR timestamp in the device driver
 * 
 *    Rev 1.1   16 Jun 2004 03:54:54   rmisalx
 * After HCT testing, code is checked in
 * 
 *    Rev 1.0   May 13 2004 14:14:14   jggoodno
 * Initial revision.
 * 
 *    Rev 1.0   14 Oct 2003 06:32:06   sgangadx
 * Initial revision.
 * Revision 1.6  2003/07/29 12:35:28Z  marcela
 * ICH_HR added
 * Revision 1.5  2003/04/01 14:57:54Z  marcela
 * add semaphore
 * Revision 1.4  2003/03/23 07:04:44Z  marcela
 * Add AMD 
 * Remove DEBUG
 * Revision 1.3  2003/01/29 10:45:34Z  marcela
 * Add ICH5
 * Revision 1.2  2002/12/19 08:04:04Z  marcela
 * Add (()) to DEBUG expressions
 * Cancel Timer - not needed
 * Revision 1.1  2002/12/11 07:27:32Z  mks
 * Initial revision
 */

#if defined(WIN32) || defined(WINVER)
    #include <wdm.h>
#endif
#include "imb_os.h"
#include "imb_if.h"
#include "imb_drv.h"
#include "smb.h"
#include "pci.h"

/////////////////////////////////////////////////////////////////////////////////////////////////
// SmbProbe
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SmbProbe
//  Purpose:    Try to find the SMB interface
//  Context:
//  Returns:    TRUE if SMB found,else  FALSE
//  Parameters:
//  Notes:      none
*F*/

static BOOL SmbProbe(void *context)
{
    UINT DeviceIndex;
    UINT FunctionIndex;
    ULONG ID;
    return PciFind(&ID, &DeviceIndex, &FunctionIndex);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// SmsBufferAvail
/////////////////////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       SmsBufferAvail
//  Purpose:    Check to see if the SMS buffer has anything in it
//  Context:
//  Returns:    TRUE ,if SMS_ATN flag set , else FALSE
//  Parameters:
//  Notes:      none
*F*/
static BOOL
SmsBufferAvail( void *context)
{
    //sms buffer is not supported
    return FALSE;
}

/*******************************************************************************
*  Routine: SMBCharact 
*  Arguments: SMBCharStruct
*  Returns:
*  Description:
*     This routine find the pci device and it's characteristics 
*******************************************************************************/
static void SMBCharact (SMBCharStruct* pSMBChar)
{
    UINT DeviceAdd;
    UINT FuncIndex;
    ULONG ID;
    
    pSMBChar->baseAddr   = 0;
    pSMBChar->Controller = 0;
    
    if (PciFind(&ID, &DeviceAdd, &FuncIndex)==FALSE)
        return;
    
    DeviceAdd = 0x80000000 | ((DeviceAdd<<11)&0xFF00);
    DeviceAdd |= ((FuncIndex<<8)&0x0700);
    
    // Read SMBus controller base address
    switch (ID) {
        case ID_81801:
        case ID_81801AA:
        case ID_81801AB:
        case ID_82801BA:
        case ID_82801CA:
        case ID_ICH4:
        case ID_ICH5:
		case ID_ICH6: //srini added to support ICH6
        case Hance_Rapids:
            pSMBChar->Controller = INTEL_SMBC;
            pSMBChar->baseAddr = (WORD)(PciReadDWord(DeviceAdd + 0x20) & 0xFFFE);
            break;
        case ID_OSB4:
        case ID_CSB5:
        case ID_CSB6:
            pSMBChar->Controller = SW_SMBC;
            pSMBChar->baseAddr = (WORD)(PciReadDWord(DeviceAdd + 0x90) & 0xFFFE);
            break;
        case ID_8111:
            // SMBus 2.0 handling
            pSMBChar->Controller = AMD_SMBC;
            pSMBChar->baseAddr = (WORD)(PciReadDWord(DeviceAdd + 0x10) & 0xFFFE);
            PciWriteDWord(DeviceAdd + AMD_SMB_BASE_2_EN, (DWORD)(1));
            break;
        default:
            break;
    } //End of Switch
    
    return;
}

//
static IMB_STATUS
SendBmcRequest (
        BmcRequest *    request,
        DWORD           reqLength,
        BmcResponse *   response,
        DWORD *         respLength,
        void *          context,
        DWORD           ipmiOldFlag
        )
{

    // Find SMBus controller and base address
    SMBCharStruct SMBchars;
    int retries = 5;
    
    SMBCharact(&SMBchars);
    
    // Check Valid Address
    if(SMBchars.baseAddr == 0)
        return IMB_SEND_ERROR;
    
    // ------------------------------------
    // Send Request - Retry 5 times at most
    // ------------------------------------
    
    do{
        if (SendRequest(request, reqLength, &SMBchars, context) == 0){
            // Success
            break;
        }
    }while (0 < retries--);
    
    if (0 >= retries){
        return IMB_SEND_ERROR;
    }
    
    // -------------------------------------
    // Read Response - Retry 5 times at most
    // -------------------------------------
    retries = 5;
    
    do{
        // Clean Response Structure
        response->cCode = response->cmd = response->data[0] = response->nfLn = 0;
        
        if ((*respLength = ReadResponse(response, &SMBchars, context))!=(-1)){
            // Success
            break;
        }
    }while (0 < retries--);
    
    if (0 >= retries){
        return IMB_SEND_ERROR;
    
    }
    
    return IMB_SUCCESS;    
}

static int
SendRequest(
    BmcRequest *    req,
    int             length,
    SMBCharStruct * smbChar,
    void *          context
    )
{
    BYTE        data;
    IpmbMsg_t * msgBuf = (IpmbMsg_t *) req;
    int         i, timeout = 0;
    os_timer_t  SendReqTimer;
    BYTE status;
    
    // Delay of 50 ms before request is sent
    Delay(50000);
    
    // Handle AMD chipset
    if (smbChar->Controller == AMD_SMBC){
        
        // SMBus 2.0 handling 
        if(amd_ec_write(smbChar->baseAddr,(BYTE)AMD_SMB_ADDR,(BYTE)(mBMCAddr & 0xFE)))
            return -1;
        if(amd_ec_write(smbChar->baseAddr,(BYTE)AMD_SMB_CMD,(BYTE)0x02))
            return -1;
        if(amd_ec_write(smbChar->baseAddr,(BYTE)AMD_SMB_BCNT,(BYTE)(length)))
            return -1;
        
        for (i = 0; i < length; i++) {
            if(amd_ec_write(smbChar->baseAddr, (BYTE)(AMD_SMB_DATA+i), *(BYTE*)((BYTE*)req + i)))
                return -1;
        } // end of for
        
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_PRTCL, AMD_SMB_BLOCK_WRITE))
            return -1;
        
        do {
            if(amd_ec_read(smbChar->baseAddr, (BYTE)AMD_SMB_STS, &data))
                return -1;
        } while ((~data & AMD_SMB_STS_DONE) && (timeout++<2000));
        
        if((~data & AMD_SMB_STS_DONE) || (data & AMD_SMB_STS_STATUS))
            return -1;
        
        
    // End of AMD Chipset Handling
    } else {
        // Handle Intel & ServerWorks Chipsets
        // Clear all status bits, Host Status Register
        switch (smbChar->Controller) {
            // Intel Status Register
            case INTEL_SMBC:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR|ICH_HST_STA_BYTE_DONE_STS;
                break;
            // ServerWorks Status Register
            case SW_SMBC:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR;
                break;
        }// End of Switch
        
        // Status Register
		// Addition by Wipro for Driver Merging Activity for ISM8.0
		// Type casting is done for resolving compilation errors for 64 bit
		// This change is done for similar kind of statements in this file.
//      WritePortUchar((((smbChar->baseAddr)+ICH_HST_STA)), status);
        WritePortUchar(  (((UCHAR*)(smbChar->baseAddr))+ICH_HST_STA), status);

        // Block protocol, Host Control Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), (BYTE)(ICH_HST_CNT_SMB_CMD_BLOCK));
        // IPMI command, Host Command Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CMD), (BYTE)(0x02));
        // Slave address, Host Address Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_XMIT_SLVA), (BYTE)(mBMCAddr));
        // Block length, Host DATA0 Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_D0), (BYTE)(length));
        // Initialize timer
        OsInitTimer(&SendReqTimer, 1000);
        
        switch (smbChar->Controller) {
            // Handle Intel Chipset
            case INTEL_SMBC:
                // the first byte
                WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_BLOCK_DB), *(BYTE *)((BYTE *)req ));
                // Block protocol and Start, Host Control Register
                WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK);
                // Send byte by byte
                for ( i = 1; i < length; i++ ) {
                    // Start timer
                    OsSetTimer(&SendReqTimer);
                    do {
                        // Read host status register
                        ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_HST_STA), &data);
                        
                        if (OsTimerTimedout(&SendReqTimer))
                            return -1;
                        
                    // Any error or Interrupt bit or byte done bit set ?
                    } while ((data & status) == 0);
                    // End Timer
                    OsCancelTimer(&SendReqTimer);
                    // Check for byte completion in block transfer
                    if ((data & ICH_HST_STA_BYTE_DONE_STS) != ICH_HST_STA_BYTE_DONE_STS)
                         break;
                    // Write next byte
                    WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_BLOCK_DB), *(BYTE *)((BYTE *)req + i));
                    // Clear status bits, Host Status Register
                    WritePortUchar(((UCHAR*)smbChar->baseAddr), (UCHAR)(data & status));
                } // End of for - Send byte by byte
            break; // End of Intel Chipset Handling
            // Handle ServerWorks Chipset
            case SW_SMBC:
                // Block length, Host DATA1 Register
                WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_D1), (BYTE)(length));
                // Reset block index
                ReadPortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), &data);
                // Put all bytes of the message to Block Data Register
                for ( i = 0; i < length; i++ )
                    WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_BLOCK_DB), *(BYTE *)((BYTE *)req + i));
                // Block protocol and Start, Host Control Register
                WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK);
                // Start timer
                OsSetTimer(&SendReqTimer);
                do {
                    // Read host status register
                    ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_HST_STA), &data);
                    
                    if (OsTimerTimedout(&SendReqTimer))
                        return -1;
                    
                } while ((data & status) == 0);    // Any error or Interrupt bit set ?
                // End Timer
                OsCancelTimer(&SendReqTimer);
            break; // End of ServerWorks Chipset Handling
        } // End of Switch Controller
        
        // Clear status bits, Host Status Register
        WritePortUchar(((UCHAR*)smbChar->baseAddr), (UCHAR)(data & status));
        
        if (data & ICH_HST_STA_ALL_ERRS)
            return -1;
         
    } // End of Intel + ServerWorks
    
    // Success
    return 0;
}

static int
ReadResponse(
    BmcResponse *  resp,
    SMBCharStruct *  smbChar,
    void * context
    )
{
    BYTE data, status, dummy, length;
    BYTE *  msgBuf = (BYTE *) resp;
    int     i, timeout =0;
    os_timer_t  SendReqTimer;
    
    // Delay current thread - time to delay in uSecs
    // 100 msec = 100000 microsec
    Delay(100000);
    
    // Handle AMD chipset
    if (smbChar->Controller == AMD_SMBC){
        // SMBus 2.0 handling 
        
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_ADDR, (BYTE)(mBMCAddr & 0xFE)))
            return -1;
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_CMD, (BYTE)0x03))
            return -1;
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_PRTCL, (BYTE)AMD_SMB_BLOCK_READ))
            return -1;
        
        do {
            if(amd_ec_read(smbChar->baseAddr, (BYTE)AMD_SMB_STS, &data))
                return -1;
        }while ((timeout++<2000)&&(~data & AMD_SMB_STS_DONE));
        
        if((~data &AMD_SMB_STS_DONE) || (data & AMD_SMB_STS_STATUS))
            return -1;
        
        if(amd_ec_read(smbChar->baseAddr, (BYTE)AMD_SMB_BCNT, &length))
            return -1;
        
        if (length > 32)    length=32;
        
        for (i = 0; i < length; i++) {
            if(amd_ec_read(smbChar->baseAddr, (BYTE)(AMD_SMB_DATA + i),&msgBuf[i]))
                return -1;
        }
        // DUMP_ATAPI expects 0 in offset 7
        if(amd_ec_write(smbChar->baseAddr, (BYTE)(AMD_SMB_DATA + 3), (BYTE)(0x00)))
            return -1;        
        
    } else {
        // Handle Intel & ServerWorks Chipsets
        switch (smbChar->Controller) {
            case INTEL_SMBC:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR|ICH_HST_STA_BYTE_DONE_STS;
                break;
            case SW_SMBC:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR;
                break;
        }// End of Switch
        
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_STA), status);
        // Block protocol, Host Control Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_SMB_CMD_BLOCK);
        // Slave address, Host Address Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_XMIT_SLVA), (BYTE)(mBMCAddr|ICH_XMIT_SLVA_READ));
        // IPMI command, Host Command Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CMD), 0x03);
        // Reset block index
        ReadPortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), &data);
        // Block protocol and Start, Host Control Register
        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK);
        // Initialize timer
        OsInitTimer(&SendReqTimer, 1000);
        // Start timer
        OsSetTimer(&SendReqTimer);
        
        do {
            // Read host status register
            ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_HST_STA), &data);
            
            if (OsTimerTimedout(&SendReqTimer))
                return -1;
            
        } while ((data & status) == 0); // Any error or Interrupt bit set ?
        // End Timer
        OsCancelTimer(&SendReqTimer);
        
        // Clear status bits, Host Status Register
        WritePortUchar(((UCHAR*)smbChar->baseAddr), (BYTE)(data & status));
        
        if (data & ICH_HST_STA_ALL_ERRS)
           return -1;
        
        // Block length, Host DATA0 Register
        ReadPortUchar((((UCHAR*)smbChar->baseAddr)+ICH_D0), &length);
        if (length >MAX_ISA_LENGTH)
            length = MAX_ISA_LENGTH;
        switch (smbChar->Controller) {
            case INTEL_SMBC:
                // Read the first byte
                ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_BLOCK_DB), &(msgBuf[0]));
                // Put all bytes of the message to Block Data Register
                for ( i = 1; i < length; i++ ) {
                    if (i == (length-1)) {
                        // Set Last Byte bit
                        ReadPortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), &dummy);
                        WritePortUchar((((UCHAR*)smbChar->baseAddr)+ICH_HST_CNT), (UCHAR)(dummy | ICH_HST_CNT_LAST_BYTE));
                    }
                    // Clear status bits, Host Status Register
                    WritePortUchar(((UCHAR*)smbChar->baseAddr), (UCHAR)(data & status));

					// Added by Wipro for Tracker no. #23939
					OsSetTimer(&SendReqTimer); // Start timer

                    do {
                        // Read host status register
                        ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_HST_STA), &data);

						// Added by Wipro for Tracker no. #23939
						if (OsTimerTimedout(&SendReqTimer))
							return -1;

                    } while ((data & status) == 0);    // Any error or Interrupt bit set ?
                    
					// Added by Wipro for Tracker no. #23939
					OsCancelTimer(&SendReqTimer); // End Timer

                    if (data & ICH_HST_STA_ALL_ERRS)
                        return -1;
                    
                    ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_BLOCK_DB), &(msgBuf[i]));
                }
                break;
            case SW_SMBC:
                // Put all bytes of the message to Block Data Register
                for ( i = 0; i < length; i++ ) {
                    ReadPortUchar(((UCHAR*)smbChar->baseAddr+ICH_BLOCK_DB), &(msgBuf[i]));
                }
                break;
        }// End of Switch
        
        if (data & ICH_HST_STA_ALL_ERRS)
            return -1;
    } // End of Intel + ServerWorks
    
    return length;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Amd
/////////////////////////////////////////////////////////////////////////////////////////////////

int amd_ec_wait_write( WORD baseAddress )
{
    int timeout = 1000;
    BYTE data;
    
    ReadPortUchar(((UCHAR*)baseAddress + AMD_EC_SC), &data);
    
    while(timeout-- && (data & AMD_EC_SC_IBF)){
        Delay(1);
        ReadPortUchar(((UCHAR*)baseAddress + AMD_EC_SC), &data);
    }
    
    if (!timeout)
        return -1;
    
    return 0;
}

int amd_ec_wait_read( WORD baseAddress )
{
    int timeout = 1000;
    BYTE data;
    
    ReadPortUchar(((UCHAR*)baseAddress + AMD_EC_SC), &data);
    
    while(timeout-- && (~data & AMD_EC_SC_OBF)){
        Delay(1);
        ReadPortUchar(((UCHAR*)baseAddress + AMD_EC_SC), &data);
    }
    
    if (!timeout)
        return -1;
    
    return 0;
}

int amd_ec_read(WORD baseAddress, BYTE address, BYTE *data)
{
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar((UCHAR*)baseAddress + AMD_EC_CMD , AMD_EC_CMD_RD);
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar((UCHAR*)baseAddress + AMD_EC_DATA , address);
    
    if (amd_ec_wait_read(baseAddress)){
        return -1;
    }
    
    ReadPortUchar(((UCHAR*)baseAddress + AMD_EC_DATA), data);
    
    return 0;
}

int amd_ec_write(WORD baseAddress, BYTE address, BYTE data)
{
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar((UCHAR*)baseAddress + AMD_EC_CMD , (BYTE)AMD_EC_CMD_WR);
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar((UCHAR*)baseAddress + AMD_EC_DATA, address);
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar((UCHAR*)baseAddress + AMD_EC_DATA, data);
    
    return 0;
}

//
// This structure is the only symbol exported by this module
// It provides the interface to discover and access the SMB if
//
ImbInterface SmbImbIf = {

    SendBmcRequest,    // sends BMC request and return response
    SmsBufferAvail,    // indicates whether the SMS buffer is available
    SmbProbe,          // indicates whether there is a SMB IF on this platform
	DEFAULT_IMB_BASE   // This Parameter is Redundant : just kept to maintain 
	                   // structure format, Shouldn't be used anywhere
};

